package MegaSquirt;
################################################################################
=head1 NAME

MegaSquirt - The MegaSquirt base class.

=head1 VERSION

Release Version: 1.1

=cut

our $VERSION = "1.1";

=head1 DATE

$Date: 2004/01/22 21:04:55 $

=head1 SYNOPSIS

  use base qw( MegaSquirt );

=head1 DESCRIPTION

The MegaSquirtclass is an abstract class should never be used directly. Instead new
classes should be derived from it.

=cut

#=============================================================================#
# Function Prototypes and Closures     #
#======================================#

#=============================================================================#
# Dependencies                         #
#======================================#

#--------------------------------------#
# Standard Dependencies
use strict;

#--------------------------------------#
# Programmatic Dependencies

#use MegaSquirt::Config qw(QA_MODE);
use constant QA_MODE => 1;
use MegaSquirt::Exception;

#=============================================================================#
# Inheritance                          #
#======================================#
use base qw();


#=============================================================================#
# Constants                            #
#======================================#


#=============================================================================#
# Fields                               #
#======================================#

#--------------------------------------#
# Public Class Fields

#--------------------------------------#
# Private Class Fields

my $gen = 'MegaSquirt::Exception::General';

#--------------------------------------#
# Public Instance Fields

use fields qw(IS_DIRTY);

#==============================================================================#
# Methods                              #
#======================================#

=head1 METHODS

=cut

########################################

=head2 Public Class Methods

=over 4

=cut

#--------------------------------------#
# Constructors

#------------------------------------------------------------------------------#
sub new {
    my __PACKAGE__ $self = fields::new(shift);
    $self->_init;
    return $self;
}

=cut

#--------------------------------------#
# Destructors


########################################

=back

=head1 PRIVATE

=head2 Private Class Methods

NONE

=head2 Private Instance Methods

=over 4

=cut

#------------------------------------------------------------------------------#

=item $self->_init

Set up default values after object construction

B<Throws:>

NONE

=cut

sub _init {
    my __PACKAGE__ $self = shift;
    $self->{IS_DIRTY} = '0';
}

#------------------------------------------------------------------------------#

=item $obj = $obj->_set(\%hash);

=item $obj = $obj->_set(\@keys, \@values);

Internal method for setting fields

B<Throws:>

=over 4

=item *

incorrect number of arguments for _set()

=item *

error setting value for '$key' in _set(): $@

=back

=cut

sub _set {
    my $self = shift;

    throw $gen({ error => 'incorrect number of args for _set()' })
	unless @_;

    my ($keys, $vals) = @_ == 1 ? ([keys %{$_[0]}], [values %{$_[0]}]) : @_;

    my ($key, $old_val, $new_val, $is_dirty);
    foreach my $i (0 .. $#$keys) {
	$key = $keys->[$i];
	$new_val = $vals->[$i];
	$old_val = $self->{$key};

	next if (not defined $new_val and not defined $old_val) or
	    (defined $new_val and defined $old_val and $old_val eq $new_val);

	# object has changed
	$is_dirty = 1;

	$self->{$key} = $new_val unless QA_MODE;

	# check for failures in QA_MODE, but this should never fail
	if (QA_MODE) {
	    eval {
		$self->{$key} = $new_val;
		$is_dirty = 1;
	    };
	    throw $gen( error => "error setting value for '$key' in _set(): $@" ) if $@;

	}	
    }
    $self->{IS_DIRTY} = 1 if $is_dirty;
    return $self;
}

=cut

#------------------------------------------------------------------------------#

=item @values || $value = $obj->_get(@keys);

The internal function used to get field values. It accepts a list of key values
to retrieve from the object.

B<Throws:>

problem retrieving field '$field': $@

B<Side Effects:>

NONE

B<Notes:>

Error checking and exception throwing is only performed in QA_MODE for
performance reasons.

=cut

sub _get {
    my $self = shift;

    # production code - no check for errors
    return wantarray ? @{$self}{@_} : $self->{$_[0]} unless QA_MODE;

    if (QA_MODE) {
	my @return;

	for (@_) {
	    # If this is a private field, we need to access it differently.
            eval { push @return, $self->{$_}};

            if ($@) {
                my $error = "problem retrieving field '$_': $@";
                throw $gen( 'error' => $error );
            }
        }

        # Syntax sugar. Let the user say $n = get_foo rather than ($n) =
        # get_foo
        return wantarray ? @return : $return[0];
    }

}

#------------------------------------------------------------------------------#

=item $obj->_is_dirty

=item $obj->_is_dirty('1');

Get or set the dirty flag

B<Throws:> NONE

B<Side Effects:> NONE

B<Notes:> NONE

=cut

sub _is_dirty {
    my $self = shift;
    $self->{IS_DIRTY} = $_[0] if @_;
    return $self->{IS_DIRTY};
}

=back

=head1 AUTHOR

steve berry, E<lt>sberry@unideal.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by steve berry

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=head1 SEE ALSO

NONE

=cut

1;
