package MegaSquirt::DataLog;
################################################################################
=head1 NAME

MegaSquirt::DataLog - The MegaSquirt class for dealing with datalogs. This
Object is created by passing a filename. The result is a collection of
DataLog::Event objects which can be iterated through using methods in this
class

=head1 VERSION

Release Version: 0.01

=cut

our $VERSION = "0.01";

=head1 DATE

$Date: 2004/01/22 21:04:55 $

=head1 SYNOPSIS

  use MegaSquirt::DataLog;

=head1 DESCRIPTION

The MegaSquirtclass is a class for dealing with VE values

=cut

################################################################################
use strict;

use MegaSquirt::DataLog::Event;
use MegaSquirt::Exception;
use Data::Dumper;

use base qw(MegaSquirt);

use constant MEM_FIELDS => qw(
			      _current_line
			      list
			      event_count
			      );

use constant FIELDS => (MEM_FIELDS);
use constant ID_FIELD => 'line_id';

my $DEBUG = 0;
my $gen = 'MegaSquirt::Exception::General';

use fields (FIELDS);

foreach my $field (ID_FIELD, FIELDS) {
    no strict 'refs';
    *$field = sub {
	my __PACKAGE__ $self = shift;
	$self->_set({ $field => shift }) if @_;
	return $self->{$field};
    }
}

sub new {
    my __PACKAGE__ $self = fields::new(shift);
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init(shift);
    return $self;
}

sub new_from_file {
    my __PACKAGE__ $self = fields::new(shift);
    my $args = shift;
    my $filename = $args->{filename} || 
	throw $gen({ error => 'Filename Required' });
    #open file and slurp VE into internal data structure
    my @lines;
    eval {
	@lines = $self->_read_file({ filename => $filename });
    };
    throw $gen({ error => "File parsing error, $@" }) if $@;

    return $self;
}

sub _init {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    $self->_set($args) if ref $args eq 'HASH';
}

sub _read_file {

# ---Developer's note
#
# Entire Datalog will be in memory waiting to be used. This sub can be refactored to read directly from the open log with seeks to reduce memory requirements without any loss of speed (benchmarks may show it to be faster)
#
# ---sberry 2004-01-25
    my $self = shift;
    my $args = shift;
    my $filename = $args->{filename};

    my @time;
    my $corrected;


    open(LOG_FILE,$filename) || die 'unable to open file ' . $filename;
    my $junk_line = (<LOG_FILE>);
    my $j = 0;
    my $i = 0;

    while (<LOG_FILE>) {
	$j++;

	my @values = split(/\t/,$_);
	push(@{$self->{list}},\@values);
	$i++;
    }
    close(LOG_FILE);
    $self->event_count($i);    
    $self->current_line(1);
}

sub current_line {
    my __PACKAGE__ $self = shift;
    my $value = shift;
    if ($value) {
	#make sure the value is in the allowed range
	die 'bad position!' 
	    unless ($value > 0 && int($value) && $value <= $self->event_count);
	$self->_current_line($value);
    }
    return $self->_current_line;
}


sub next {
    my __PACKAGE__ $self = shift;
    return undef if $self->current_line == $self->event_count;
    $self->current_line($self->current_line + 1);
    return $self->event;
}

sub previous {
    my __PACKAGE__ $self = shift;
    return undef if $self->current_line < 1;
    $self->current_line($self->current_line - 1);
    return $self->event;
}

sub event {
    my __PACKAGE__ $self = shift;
    my $data = $self->list->[$self->current_line - 1];
    return MegaSquirt::DataLog::Event->new({ line => $data,
					     line_id => $self->current_line });
}

sub old_event {
    my __PACKAGE__ $self = shift;
    return $self->list->[$self->current_line - 1];
}

sub delete {
    my __PACKAGE__ $self = shift;
}

sub save {
    my __PACKAGE__ $self = shift;
    if ($self->_get(ID_FIELD)) {
	$self->_update;
    } else {
	$self->_insert;
    }
}


sub _list {
    my ($class, $args, $ids_only) = @_;
}

sub _insert {
    my __PACKAGE__ $self = shift;
}

sub _update {
    my __PACKAGE__ $self = shift;
    return unless $self->_is_dirty;
}


=head1 AUTHOR

steve berry, E<lt>sberry@unideal.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by steve berry

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=head1 SEE ALSO

NONE

=cut

1;
