package MegaSquirt::VE;
################################################################################
=head1 NAME

MegaSquirt::VE - The MegaSquirt class for deling with ve tables

=head1 VERSION

Release Version: 0.01

=cut

our $VERSION = "0.01";

=head1 DATE

$Date: 2004/01/22 21:04:55 $

=head1 SYNOPSIS

  use MegaSquirt::VE;

=head1 DESCRIPTION

The MegaSquirtclass is a class for dealing with VE values

=cut

################################################################################
use strict;

use MegaSquirt::Exception;
use Data::Dumper;

use base qw(MegaSquirt);

use constant MEM_FIELDS => qw(
			      test
			      filename
			      _matrix
			      _rpms
			      _kpas
			      );

use constant FIELDS => (MEM_FIELDS);
use constant ID_FIELD => 've_id';

my $MAX_INDEX = 7;
my $DEBUG = 0;
my $gen = 'MegaSquirt::Exception::General';

use fields (FIELDS);


my $VALIDATION = {
    ve_bin => { required => 1 },
};

sub validate {
    my __PACKAGE__ $self = fields::new(shift);
    my ($field,$val) = @_;
    
    if ($VALIDATION->{$field}->{required}) {
	return 'Field required' unless $val;
    } else {
	return undef unless $val;
    }

    if (defined $VALIDATION->{$field}->{method}) {
	my $rv;
	foreach my $method (@{$VALIDATION->{$field}->{method}}) {
	    $rv = $self->$method($val);
	    return $rv if $rv;
	}
	return $rv;
    }
    return undef;
}

foreach my $field (ID_FIELD, FIELDS) {
    no strict 'refs';
    *$field = sub {
	my __PACKAGE__ $self = shift;
	$self->_set({ $field => shift }) if @_;
	return $self->{$field};
    }
}

sub new {
    my __PACKAGE__ $self = fields::new(shift);
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init(shift);
    return $self;
}


sub nearest_bin {
    my __PACKAGE__ $self = shift;
    my $rpm = shift;
    my $kpa = shift;
    #   warn qq($rpm, $kpa);
#    print Dumper($self->_rpms);
    my $rpm_bin = $self->nearest($rpm,$self->_rpms);
    my $kpa_bin = $self->nearest($kpa,$self->_kpas);
    return ($rpm_bin, $kpa_bin);
}

sub nearest {
    my __PACKAGE__ $self = shift;
    my $value = shift;
    my $array = shift;

    my $i = 0;
    foreach (@$array) { 
	if ($_ < $value) {
	    $i++;
	    next;
	}
	return $array->[$i] || $array->[$i - 1];
#	print "return $_ if $value >= $_; ";
    }
    return $array->[-1];
}

sub new_from_file {
    my __PACKAGE__ $self = fields::new(shift);
    my $args = shift;
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init($args);
    
    my $filename = $args->{filename} || 
	throw $gen({ error => 'Filename Required' });

    #open file and slurp VE into internal data structure
    eval {
	$self->_read_file({ filename => $filename });
    };

    print $@ if $@;
    throw $gen({ error => "File parsing error, $@" }) if $@;

    $self->_is_dirty('0');
    
    return $self;
}

sub _init {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    $self->_set($args) if ref $args eq 'HASH';
}

sub _read_file {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    my $filename = $args->{filename};

    open(VEX_FILE,$filename) || die 'unable to open file ' . $filename;

    my @lines;
    
    while (<VEX_FILE>) {
	push(@lines,$_);
    }
    close VEX_FILE;
    my @rpm;
    for (7 .. 14) {
	$lines[$_] =~ m/\=\s*(\d*)\s/;
	push(@rpm,$1);
    }
    my @kpa;
    for (16 .. 23) {
	$lines[$_] =~ m/\=\s*(\d*)\s/;
	push(@kpa,$1);
    }
    
    my @rows;
    for (26 .. 33) {
	$lines[$_] =~ m/\=\s*(\d.*\d)\s/;	
	my $row = $1;
	my @vals = split(/\s\s*/,$row);
	push(@rows,\@vals);
    }

    # this is just vanity
    # flip the matrix for rpm=x,kpa=y ($m->[rpm]->[kpa])
    my $struc = $self->_flip_matrix(\@rows);;
    eval {
	$self->load_matrix({ rpms => \@rpm,
			     kpas => \@kpa,
			     rows => $struc,
			 });
    };
    warn $@ if $@;
}

sub _flip_matrix {
    my $self = shift;
    my $struc = shift;
    my $new_struc;
    # flip the axis
    for my $val1 (0 .. $MAX_INDEX) {
	for my $val2 (0 .. $MAX_INDEX) {
	    $new_struc->[$val1]->[$val2] = $struc->[$val2]->[$val1];
	}
    }
    return $new_struc;
}

sub pad {
    my $val = shift;
    my $pad = shift;
    for (length($val) .. $pad) { 
	 $val = ' ' . $val;
     }
    return $val;
}

sub dump_matrix {
    my __PACKAGE__ $self = shift;

    my $flipped = $self->_flip_matrix($self->_matrix);
    my @rpms;
    foreach (@{$self->_rpms}) {
	push(@rpms, pad($_,1));
    }
    
    my $out;
    $out .= '      ' . join('--',@rpms) . "\n";
    $out .= "     --------------------------------\n";
    my $i = 0;
    foreach my $row (@{$flipped}) {
	$out .= pad($self->_kpas->[$i],3) . "|";
	foreach my $val (@$row) {
	    $out .= "-$val-";
	}
	$out .= "\n";
	$i++;
    }
    $out .= "\n";

}

sub load_matrix {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    # verify that the rpm and kpa numbers make sense
    eval { $self->legal_bin_labels($args->{rpms}); };
    eval { $self->legal_bin_labels($args->{kpas}); };
    throw $gen({ error => "$@" }) if $@;

    $self->_rpms($args->{rpms});
    $self->_kpas($args->{kpas});
    $self->_matrix($args->{rows}); 


}

sub legal_bin_labels {
    my __PACKAGE__ $self = shift;
    my $self = shift;
    my $arr = shift;
    my $i;
    foreach (@$arr) {
	throw $gen({ error => "$@" }) if $@;
	$i = $_;
    }
}

sub bin {
    my __PACKAGE__ $self = shift;
    my ($rpm, $kpa, $is_index, $set) = @_;

    my $rpm_index = $rpm;
    my $kpa_index = $kpa;
    unless ($is_index) {
	$rpm_index = _in($rpm,$self->_rpms);
	$kpa_index = _in($kpa,$self->_kpas);
    }
    return undef unless defined($rpm_index);
    return undef unless defined($kpa_index);
    if(defined($set)) {
#	warn 'HERE';
	$self->_matrix->[$rpm_index]->[$kpa_index] = $set;
    }
    return $self->_matrix->[$rpm_index]->[$kpa_index];
}

sub _in {
    my $value = shift;
    my $array = shift;
    my $i = 0;
    for ($i .. ($array->[-1])) { return $i if $value eq $array->[$i]; $i++; }
    return undef;
}

sub rpm {
    my __PACKAGE__ $self = shift;
    my $rpm_index = shift;
    return $self->_rpms->[$rpm_index];
}

sub kpa {
    my __PACKAGE__ $self = shift;
    my $kpa_index = shift;
    return $self->_kpas->[$kpa_index];
}

sub calculate_ve {
    my __PACKAGE__ $self = shift;
    my $args = shift;

    my $rpm = $args->{rpm};
    my $kpa = $args->{kpa};

    return $self->bin($rpm,$kpa) if $self->bin($rpm,$kpa);

    my ($brpm,$bkpa);
    ($brpm->{1},$brpm->{2}) = $self->bracketed($args->{rpm},$self->_rpms);
    ($bkpa->{1},$bkpa->{2}) = $self->bracketed($args->{kpa},$self->_kpas);

    #########################################################
    if ($DEBUG) {
    print qq( rpm->{1} = $brpm->{1}, rpm->{2} = $brpm->{2});
    print qq( kpa->{1} = $bkpa->{1}, kpa->{2} = $bkpa->{2});
    }
    #########################################################


    #########################################################
    ## x1=rpm1, x2=rpm2, y1=VE(kpa1,rpm1), y2=VE(kpa1,rpm2)
    if ($DEBUG) {
    print qq(\n---------------------------\n);
    print qq(x1=rpm1, x2=rpm2, y1=VE(kpa1,rpm1), y2=VE(kpa1,rpm2)\n);
    print "x = $rpm\n";
    print "x1 = " . $self->rpm($brpm->{1}) . "\n";
    print "x2 = " . $self->rpm($brpm->{2}) . "\n";
    print "y1 = " . $self->bin($brpm->{1},$bkpa->{1},1) . "\n";
    print "y2 = " . $self->bin($brpm->{2},$bkpa->{1},1) . "\n";
    }
    #########################################################



    my $ve_low_kpa = $self->_interpolate({ 
	x => $rpm,
	x1 => $self->rpm($brpm->{1}),
	x2 => $self->rpm($brpm->{2}),
	y1 => $self->bin($brpm->{1},$bkpa->{1},1),
	y2 => $self->bin($brpm->{2},$bkpa->{1},1),
    });
    
    #########################################################
    ## x1=rpm1, x2=rpm2, y1=VE(kpa2,rpm1), y2=VE(kpa2,rpm2)
    if ($DEBUG) {
    print qq(\n---------------------------\n);
    print qq(x1=rpm1, x2=rpm2, y1=VE(kpa2,rpm1), y2=VE(kpa2,rpm2)\n);
    print "x = $rpm\n";
    print "x1 = " . $self->rpm($brpm->{1}) . "\n";
    print "x2 = " . $self->rpm($brpm->{2}) . "\n";
    print "y1 = " . $self->bin($brpm->{1},$bkpa->{2},1) . "\n";
    print "y2 = " . $self->bin($brpm->{2},$bkpa->{2},1) . "\n";
    }
    #########################################################

    my $ve_high_kpa = $self->_interpolate({ 
	x => $rpm,
	x1 => $self->rpm($brpm->{1}),
	x2 => $self->rpm($brpm->{2}),
	y1 => $self->bin($brpm->{1},$bkpa->{2},1),
	y2 => $self->bin($brpm->{2},$bkpa->{2},1)
	});

    #########################################################
    ## x1=kpa1, x2=kpa2, y1=VE_FROM_STEP_4, y2=VE_FROM_STEP_5
    if ($DEBUG) {
    print qq(\n---------------------------\n);
    print qq(x1=kpa1, x2=kpa2, y1=VE_FROM_STEP_4, y2=VE_FROM_STEP_5\n);
    print "x = $kpa\n";
    print "x1 = " . $self->kpa($bkpa->{1}) . "\n";
    print "x2 = " . $self->kpa($bkpa->{2}) . "\n";
    print "y1 = " . $ve_low_kpa . "\n";
    print "y2 = " . $ve_high_kpa . "\n";
    }
    #########################################################

    my $ve = $self->_interpolate({  x => $kpa,
				    x1 => $self->kpa($bkpa->{1}),	
				    x2 => $self->kpa($bkpa->{2}),
				    y1 => $ve_low_kpa,
				    y2 => $ve_high_kpa, });
    
    return $ve;
}

sub _interpolate {
    # linear
    my $self = shift;
    my $a = shift;
    my $Y;
    if ($DEBUG) {
    print Dumper($a);
    print <<END;
	my $Y = 
	    $a->{y1} + 
	    
	    ($a->{y2} - $a->{y1})
	    /
	    ($a->{x2} - $a->{x1}) +
	    
	    ($a->{x} - $a->{x1});
    };
END
    }

    $Y;
    eval {
# based on: Y = y1 + (y2 - y1)/(x2 - x1) + (x - x1)
	$Y = 
	    $a->{y1} + 
	    
	    ($a->{y2} - $a->{y1})
	    /
	    ($a->{x2} - $a->{x1}) +
	    
	    ($a->{x} - $a->{x1});
    };
    $Y = 0 if $@ =~ 'Illegal division by zero';
print "Y = $Y\n" if $DEBUG;
    return $Y;
}

sub bracketed {
    my __PACKAGE__ $self = shift;
    if ($DEBUG) {
	print "\n----------------------\n";
    }
    my $value = shift;
    my $array = shift;
    
    my $i = 0;
    my ($low, $high);
    return ($MAX_INDEX,$MAX_INDEX) if $value > $array->[$MAX_INDEX];

    
    for ($i = 0; $i <= $MAX_INDEX; $i++) {
	if ($DEBUG) {
	    print qq((i: $i, $array->[$i], value: $value)\n);
	}
	next unless (($value >= $array->[$i]) && ($value <= $array->[$i + 1]));
	print "$i next unless (($value > $array->[$i]) && ($value < $array->[$i + 1]))" if $DEBUG;
#	print "$i next unless $value > $array->[$i]\n";
	$low = $i;
	$high = $i + 1;
	print "\nFOUND (low: $low, high: $high)\n" if $DEBUG;
#	print "next unless $value > $array->[$i]\n";
	return ($low, $high);
    }
}

sub bin_ve {
    my $self = shift;
    my $args = shift;
    

}




sub lookup {
    my __PACKAGE__ $self = fields::new(shift);
    my $args = shift;
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init($args);

    $self->_is_dirty('0');
    return $self;
}


sub list {
    my ($class, $args) = @_;
    return _list($class, $args, undef);
}

sub list_ids {
    my ($class, $args) = @_;
    return _list($class, $args, 1);
}

sub delete {
    my __PACKAGE__ $self = shift;
}

sub save {
    my __PACKAGE__ $self = shift;
    if ($self->_get(ID_FIELD)) {
	$self->_update;
    } else {
	$self->_insert;
    }
}




sub _list {
    my ($class, $args, $ids_only) = @_;
}

sub _insert {
    my __PACKAGE__ $self = shift;
}

sub _update {
    my __PACKAGE__ $self = shift;
    return unless $self->_is_dirty;
}


#sub _in {
#    my $value = shift;
#    my $array = shift;
#    my $i = 0;
#    while (@$array) { return $i if $value eq $_; $i++; }
#    return 0;
#}

sub voltage_to_af {
    my $self = shift;
    my $voltage = shift;
    return ((2* $voltage) + 10);
}

sub af_to_voltage {
    my $self = shift;
    my $af = shift;
    return 0 unless $af;
    return $af/2 -10;
}


=head1 AUTHOR

steve berry, E<lt>sberry@unideal.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by steve berry

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=head1 SEE ALSO

NONE

=cut

1;
