package MegaSquirt::Stats;
################################################################################
=head1 NAME

MegaSquirt::Stats - The MegaSquirt class for dealing with stats

=head1 VERSION

Release Version: 0.01

=cut

our $VERSION = "0.01";

=head1 DATE

$Date: 2004/01/22 21:04:55 $

=head1 SYNOPSIS

  use MegaSquirt::Stats;

=head1 DESCRIPTION

Simple stats operations

=cut

################################################################################
use strict;

use MegaSquirt::DataLog::Event;
use MegaSquirt::Exception;
use Data::Dumper;

use base qw(MegaSquirt);

use constant MEM_FIELDS => qw(
			      _current_value
			      list
			      max_index
			      min_index
			      median
			      outlier_index_low
			      outlier_index_high
			      mean
			      quartile1
			      quartile2
			      quartile3
			      quartile4
			      _outliers_off
			      );

use constant FIELDS => (MEM_FIELDS);
use constant ID_FIELD => 'index';

my $DEBUG = 0;
my $gen = 'MegaSquirt::Exception::General';

use fields (FIELDS);

foreach my $field (ID_FIELD, FIELDS) {
    no strict 'refs';
    *$field = sub {
	my __PACKAGE__ $self = shift;
	$self->_set({ $field => shift }) if @_;
	return $self->{$field};
    }
}

sub new {
    my __PACKAGE__ $self = fields::new(shift);
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init(shift);
    return $self;
}

sub remove_outliers {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    
}


sub set_quartiles {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    my @list = $self->list;
    
}

sub calculate_mean {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    $self->mean($self->avg($self->list));
}

sub calculate_median {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    my $median_index;
    my @list = @{$self->list};
    $median_index = (($self->max_index - $self->min_index) / 2);

    if ($self->max_index % 2) {
	$self->median($self->avg([$list[$median_index], $list[$median_index + 1]]));
    } else {
	$self->median($list[$median_index]);
    }

}

sub _init {
    my __PACKAGE__ $self = shift;
    my $args = shift;
    $self->_set($args) if ref $args eq 'HASH';
    $args->{list} ||= [0];
    my @list = @{$args->{list}};
    
    $self->remove_outliers unless $self->_outliers_off;

    $self->list(\@list);
    $self->max_index($#list);
    $self->min_index(0);
    return unless $self->max_index;

    $self->calculate_median;
    $self->calculate_mean;

}




sub avg {
    my __PACKAGE__ $self = shift;
    my $numbers = shift;
    my $sum = 0;
    my $i = 0;
    foreach (@$numbers) {
	$i++;
	$sum += $_;
    }
    return $sum/$i;
}

sub next {
    my __PACKAGE__ $self = shift;
    return undef if $self->current_line == $self->event_count;
    $self->index($self->index + 1);
    return $self->event;
}

sub previous {
    my __PACKAGE__ $self = shift;
    return undef if $self->current_line < 1;
    $self->index($self->index - 1);
}

sub event {
    my __PACKAGE__ $self = shift;
    my $data = $self->list->[$self->current_line - 1];
    return MegaSquirt::DataLog::Event->new({ line => $data,
					     line_id => $self->current_line });
}

sub old_event {
    my __PACKAGE__ $self = shift;
    return $self->list->[$self->current_line - 1];
}

sub delete {
    my __PACKAGE__ $self = shift;
}

sub save {
    my __PACKAGE__ $self = shift;
    if ($self->_get(ID_FIELD)) {
	$self->_update;
    } else {
	$self->_insert;
    }
}


=head1 AUTHOR

steve berry, E<lt>sberry@unideal.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by steve berry

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=head1 SEE ALSO

NONE

=cut

1;
