package MegaSquirt::DataLog::Event;
################################################################################
use lib '/home/sberry/src/megasquirt/lib';
=head1 NAME

MegaSquirt::DataLog::Event - The MegaSquirt class for dealing with datalogs

=head1 VERSION

Release Version: 0.01

=cut

our $VERSION = "0.01";

=head1 DATE

$Date: 2004/01/22 21:04:55 $

=head1 SYNOPSIS

  Called from MegaSquirt::DataLog;
  Do not call directly

=head1 DESCRIPTION

The Event class is a class for dealing with DataLog entires

=cut

################################################################################
use strict;

use MegaSquirt::Exception;
use Data::Dumper;

use base qw(MegaSquirt);

use constant MEM_FIELDS => qw(
			      afr
			      nve
			      rpm_print
			      );

use constant LOG_FIELDS => qw(
			      time 
			      sec 
			      rpm 
			      kpa 
			      tp 
			      o2 
			      mat 
			      clt 
			      engine 
			      gego 
			      gair 
			      gwarm 
			      gbaro 
			      gammae 
			      tpsacc 
			      gve 
			      pw 
			      gve2 
			      pw2 
			      dutycycle1 
			      dutycycle2 
			      userdata1 
			      userdata2 
			      userdata3
			      );




use constant ID_FIELD => 'line_id';
use constant FIELDS => (ID_FIELD,MEM_FIELDS,LOG_FIELDS);


my $MAX_INDEX = 7;
my $DEBUG = 0;
my $gen = 'MegaSquirt::Exception::General';

use fields (FIELDS);

foreach my $field (ID_FIELD, FIELDS, LOG_FIELDS) {
    no strict 'refs';
    *$field = sub {
	my __PACKAGE__ $self = shift;
	$self->_set({ $field => shift }) if @_;
	return $self->{$field};
    }
}

sub new {
    my __PACKAGE__ $self = fields::new(shift);
    $self->SUPER::new();
    $self->SUPER::_init();
    $self->_init(shift);
    return $self;
}


sub _init {
    my __PACKAGE__ $self = shift;
    my $args = shift;
#    die 'No data!' unless $args->{line};
    my $line = $args->{line};
    $self->line_id($args->{line_id});
    my $args_hash;
    
    my $i = 0;
    foreach my $col (LOG_FIELDS) {
	$args_hash->{$col} = $line->[$i];
	$i++;
    }
    $args_hash->{afr} = voltage_to_af($args_hash->{o2});
    $args_hash->{nve} = round($self->correct_ve($args_hash->{gve},$args_hash->{afr}));
    $args_hash->{rpm_print} = 100 * $args_hash->{rpm};

    $self->_set($args_hash) if ref $args_hash eq 'HASH';
    
    if ($DEBUG) {
	print qq(afr: $args_hash->{afr}\n);
	print qq(gve: $args_hash->{gve}\n);
#	print qq(clc: ) . $ve->calculate_ve({ rpm => $args_hash->{rpm}, kpa => $args_hash->{kpa} }) . qq(\n);
	print qq(kpa: $args_hash->{kpa}\n);
	print qq(rpm: ) . 100 * ($args_hash->{rpm}) .qq(\n);
	print qq(nve: $args_hash->{nve}\n);
    }
    
#    my ($rpm_bin, $kpa_bin) = $ve->nearest_bin($args_hash->{rpm},$args_hash->{kpa});
    
#    print qq(affects RPM: $rpm_bin, KPA: $kpa_bin\n) if $DEBUG;
    print "     --------------------------------\n" if $DEBUG;
    
#    push(@{$corrected->{$rpm_bin}->{$kpa_bin}}, $args_hash->{nve});
}

sub round {
    my $val = shift;
    $val =~ /(.*)\.(\d)/;
    return $1 unless $2;

    return $1 + 1 if $2 >= 5;
    return $1;
}

sub voltage_to_af {
    my $voltage = shift;
    return ((2* $voltage) + 10);
}

sub af_to_voltage {
    my $af = shift;
    return 0 unless $af;
    return $af/2 -10;
}

sub correct_ve {
    my __PACKAGE__ $self = shift;
    my $ve = shift;
    my $afr = shift;
    my $afr_target = shift || 14.5;
    return 'NULL' unless $afr;
    my $new_ve = ($ve * ($afr / $afr_target));

    return $new_ve;

}

sub list_ids {
    my ($class, $args) = @_;
    return _list($class, $args, 1);
}

sub delete {
    my __PACKAGE__ $self = shift;
}

sub save {
    my __PACKAGE__ $self = shift;
    if ($self->_get(ID_FIELD)) {
	$self->_update;
    } else {
	$self->_insert;
    }
}


sub _list {
    my ($class, $args, $ids_only) = @_;
}

sub _insert {
    my __PACKAGE__ $self = shift;
}

sub _update {
    my __PACKAGE__ $self = shift;
    return unless $self->_is_dirty;
}


=head1 AUTHOR

steve berry, E<lt>sberry@unideal.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by steve berry

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=head1 SEE ALSO

NONE

=cut

1;
