use strict;
use MegaSquirt::VE;
use Data::Dumper;
our $PRINT = 0;
push (@INC,'/home/sberry/msquirt/msquirt');

my $log_file = $ARGV[0];
die 'need log file' unless $log_file;

my $vex_file = $ARGV[1];
die 'need vex file' unless $vex_file;

my $ve;
eval {
    $ve = MegaSquirt::VE->new_from_file({ filename => $vex_file });
};

my $original = $ve->dump_matrix;
my $events = parse_log_file($log_file, $ve);


sub parse_log_file {
    my $log_file = shift;
    my $ve = shift;
    open(FILE,$log_file);
    my @cols = qw(time sec rpm kpa tp o2 mat clt engine gego gair gwarm gbaro gammae tpsacc gve pw gve2 pw2 dutycycle1 dutycycle2 userdata1 userdata2 userdata3);
    my @time;
    my $corrected;
    my $j = 1;
    while (<FILE>) {
	$j++;
#	next if $j > 395;
	next if $j > 5000;
	my @values = split(/\t/,$_);
	push(@time,\@values);
    }
    close(FILE);

    my @rv;
    LINE: foreach my $second (@time) {
	
	next unless $second->[2] > 11;
#	print $ve->dump_matrix if ($PRINT);
	my $i = 0;
	my $hash;
	foreach my $col (@cols) {
	    $hash->{$col} = $second->[$i];
#	    next LINE unless $hash->{rpm} > 5;
#	    print qq($col: $second->[$i]\n);
	    $i++;
	}
	$hash->{afr} = voltage_to_af($hash->{o2});
	my $PRINT = 1 if $hash->{afr} > 15.5;
	
	$hash->{nve} = round(correct_ve($hash->{gve},$hash->{afr}));

	if ($PRINT) {
	print qq(afr: $hash->{afr}\n);
	print qq(gve: $hash->{gve}\n);
	print qq(clc: ) . $ve->calculate_ve({ rpm => $hash->{rpm}, kpa => $hash->{kpa} }) . qq(\n);
	print qq(kpa: $hash->{kpa}\n);
	print qq(rpm: ) . 100 * ($hash->{rpm}) .qq(\n);
	print qq(nve: $hash->{nve}\n);
    }

	my ($rpm_bin, $kpa_bin) = $ve->nearest_bin($hash->{rpm},$hash->{kpa});

	print qq(affects RPM: $rpm_bin, KPA: $kpa_bin\n) if $PRINT;
	print "     --------------------------------\n" if $PRINT;

	push(@{$corrected->{$rpm_bin}->{$kpa_bin}}, $hash->{nve});
	push (@rv, $hash);
    }


    process_corrected($ve, $corrected);
}
    

sub process_corrected {
    my $ve = shift;
    my $corrected = shift;


    foreach my $rpm (keys %$corrected) {
	foreach my $kpa (keys %{$corrected->{$rpm}}) {
#	    warn 'HERE';
	    my $i = 0;
	    my $sum;
	    foreach my $val (sort(@{$corrected->{$rpm}->{$kpa}})) {
		$i++;
		print "$val, ";
		$sum += $val;
	    }
	    my $avg = round($sum/$i);
	    print "\n\nAVG: $avg\n\n";
	    $ve->bin($rpm,$kpa,0,$avg);
#	    $ve->bin($rpm,$kpa,0,1);
	}
    }
    print "ORIGINAL\n$original\n\n-----------------------------\n";
    print "NEW\n" . $ve->dump_matrix . "\n";
}

sub round {
    my $val = shift;
    $val =~ /(.*)\.(\d)/;
    return $1 unless $2;

    return $1 + 1 if $2 >= 5;
    return $1;
}

sub voltage_to_af {
    my $voltage = shift;
    return ((2* $voltage) + 10);
}

sub af_to_voltage {
    my $af = shift;
    return 0 unless $af;
    return $af/2 -10;
}

sub correct_ve {
    my $ve = shift;
    my $afr = shift;
    return 'NULL' unless $afr;
    return ($ve * ($afr / 14.3))
}
