use strict;

use MegaSquirt::VE;
use MegaSquirt::DataLog;
use Data::Dumper;
use Statistics::Descriptive;

our $PRINT = 0;

my $log_file = $ARGV[0];
die 'need log file' unless $log_file;

my $vex_file = $ARGV[1];
die 'need vex file' unless $vex_file;

my $log;
eval {
    $log = MegaSquirt::DataLog->new_from_file({ filename => $log_file });
};
warn $@ if $@;
my $ve;
eval {
    $ve = MegaSquirt::VE->new_from_file({ filename => $vex_file });
};
 
my $original = $ve->dump_matrix;

my $lowest_kpa = 100;
my $highest_kpa = 0;
my $lowest_rpm = 100;
my $highest_rpm = 0;

my $corrected;
while (my $event = $log->next) {
#    next unless $event->kpa < 20;
    my $calc = $ve->calculate_ve({ rpm => $event->rpm,
				   kpa => $event->kpa, });
#    next unless $calc == 0;
    
    my ($rpm_bin, $kpa_bin) = $ve->nearest_bin($event->rpm,$event->kpa);
    my $afr = $event->{afr};
#    next if ($afr < 15 && $afr > 11.5);
    my $rpm = $event->rpm;
    my $kpa = $event->kpa;
    my $gve = $event->gve;
    my $nve = $event->nve;
    my $afr_target = afr_target($rpm,$kpa);
    $nve = $event->correct_ve($event->gve,$event->afr,$afr_target);# unless $nve;
    my $change = $nve - $event->gve;
    push(@{$corrected->{$rpm_bin}->{$kpa_bin}->{nve_data}}, $nve) if $nve;
    my $sign = $change > 0 ? '+' : '';

    $lowest_kpa = $event->kpa if $event->kpa < $lowest_kpa;
    $highest_kpa =  $event->kpa if $event->kpa > $highest_kpa;
    $lowest_rpm = $event->rpm if $event->rpm < $lowest_rpm;
    $highest_rpm = $event->rpm if $event->rpm > $highest_rpm;

    print qq(RPM: $rpm_bin ($rpm), KPA: $kpa_bin ($kpa) AFR: $afr  CALC: $calc VE: $gve NEW_VE: $nve ($sign$change)\n) if (int($change) > 10 || int($change) < -10);
#   print qq(RPM: $rpm_bin ($rpm), KPA: $kpa_bin ($kpa) AFR: $afr  CALC: $calc VE: $gve NEW_VE: $nve ($sign$change)\n);
#	print "     --------------------------------\n" if $PRINT;

#	push(@{$corrected->{$rpm_bin}->{$kpa_bin}}, $hash->{nve});
#	push (@rv, $hash);
}


process_corrected($ve, $corrected);

print "LOW KPA: $lowest_kpa\n";
print "HI  KPA: $highest_kpa\n";
print "LOW RPM: $lowest_rpm\n";
print "HI  RPM: $highest_rpm\n";

print $ve->out;
    
sub process_corrected {
    my $ve = shift;
    my $corrected = shift;

    use Statistics::Lite qw(:all);

    foreach my $rpm (sort(keys %$corrected)) {
	foreach my $kpa (sort(keys %{$corrected->{$rpm}})) {
#	    warn 'HERE';
	    my $i = 0;
	    my $sum;
	    foreach my $val (sort(@{$corrected->{$rpm}->{$kpa}->{nve_data}})) {
		$i++;
#		print "$val, ";
		$sum += $val;
	    }
	    my $avg = round($sum/$i);
	    my %stats_hash = statshash(@{$corrected->{$rpm}->{$kpa}->{nve_data}});
	    my $rprint = 100 * $rpm;

	    print Dumper(\%stats_hash) . qq(\n);
	    


	    print "\n\nrpm: $rprint, kpa: $kpa == AVG: $avg\n\n";
	    $ve->bin($rpm,$kpa,0,$avg);
#	    $ve->bin($rpm,$kpa,0,1);
	}
    }
    print "ORIGINAL\n$original\n\n-----------------------------\n";
    print "NEW\n" . $ve->dump_matrix . "\n";
}

sub afr_target {
    my $rpm = shift;
    my $kpa = shift;
#    print "THISRPM: $rpm\n";
    return 12.9 if $kpa >= 50;
    return 12.9 if $rpm >= 36;
    return 14.5;
}


sub round {
    my $val = shift;
    $val =~ /(.*)\.(\d)/;
    return $1 unless $2;

    return $1 + 1 if $2 >= 5;
    return $1;
}

__END__




sub voltage_to_af {
    my $voltage = shift;
    return ((2* $voltage) + 10);
}

sub af_to_voltage {
    my $af = shift;
    return 0 unless $af;
    return $af/2 -10;
}

sub correct_ve {
    my $ve = shift;
    my $afr = shift;
    return 'NULL' unless $afr;
    return ($ve * ($afr / 14.3))
}
