use strict;

use MegaSquirt::VE;
use MegaSquirt::DataLog;
use Data::Dumper;
our $PRINT = 0;

my $vex_file = $ARGV[0];
die 'need vex file' unless $vex_file;


my $ve;

eval {
    $ve = MegaSquirt::VE->new_from_file({ filename => $vex_file });
};
print "EVEME: " . $ve->EVEME . "\n";
print "UserRev: " . $ve->UserRev . "\n";
print "UserComment: " . $ve->UserComment . "\n";
print "Date: " . $ve->Date . "\n";
print "Time: " . $ve->Time . "\n";
print "Page: " . $ve->Page . "\n";
print "RPM RANGE: " . $ve->rpm_range . "\n";
print "RPM LABELS: " . "@{$ve->{_rpms}}" . "\n";
print "KPA RANGE: " . $ve->kpa_range . "\n";
print "KPA LABELS: " . "@{$ve->{_kpas}}" . "\n";
print "RPM MAX: " . $ve->max_rpm_index . "\n";
print "KPA MAX: " . $ve->max_kpa_index . "\n";
print $ve->dump_matrix;

print $ve->out;
__END__
 
my $original = $ve->dump_matrix;

while (my $event = $log->next) {
    my ($rpm_bin, $kpa_bin) = $ve->nearest_bin($event->rpm,$event->kpa);
    my $afr = $event->{afr};
    next if $afr < 15;
    my $rpm = $event->rpm;
    my $kpa = $event->kpa;
    my $gve = $event->gve;
    my $nve = $event->nve;
    my $change = $nve - $event->gve;
    my $sign = $change > 0 ? '+' : '';

    print qq(RPM: $rpm_bin ($rpm), KPA: $kpa_bin ($kpa) AFR: $afr  VE: $gve NEW_VE: $nve ($sign$change)\n);
#	print "     --------------------------------\n" if $PRINT;

#	push(@{$corrected->{$rpm_bin}->{$kpa_bin}}, $hash->{nve});
#	push (@rv, $hash);
}


#    process_corrected($ve, $corrected);

    
__END__
sub process_corrected {
    my $ve = shift;
    my $corrected = shift;


    foreach my $rpm (keys %$corrected) {
	foreach my $kpa (keys %{$corrected->{$rpm}}) {
#	    warn 'HERE';
	    my $i = 0;
	    my $sum;
	    foreach my $val (sort(@{$corrected->{$rpm}->{$kpa}})) {
		$i++;
		print "$val, ";
		$sum += $val;
	    }
	    my $avg = round($sum/$i);
	    print "\n\nAVG: $avg\n\n";
	    $ve->bin($rpm,$kpa,0,$avg);
#	    $ve->bin($rpm,$kpa,0,1);
	}
    }
    print "ORIGINAL\n$original\n\n-----------------------------\n";
    print "NEW\n" . $ve->dump_matrix . "\n";
}

sub round {
    my $val = shift;
    $val =~ /(.*)\.(\d)/;
    return $1 unless $2;

    return $1 + 1 if $2 >= 5;
    return $1;
}

sub voltage_to_af {
    my $voltage = shift;
    return ((2* $voltage) + 10);
}

sub af_to_voltage {
    my $af = shift;
    return 0 unless $af;
    return $af/2 -10;
}

sub correct_ve {
    my $ve = shift;
    my $afr = shift;
    return 'NULL' unless $afr;
    return ($ve * ($afr / 14.3))
}
